@@ -34,6 +34,7 @@
 #include "zenity.h"
 #include "util.h"
 
+#ifdef HAVE_LIBNOTIFY
 static char *icon_file;
 
 static void
@@ -132,6 +133,7 @@ zenity_notification_handle_stdin (GIOChannel *channel,
           if (error) {
             g_warning ("Error showing notification: %s", error->message);
             g_error_free (error);
+            error = NULL;
           }
 
 	  g_object_unref (notif);
@@ -149,6 +151,7 @@ zenity_notification_handle_stdin (GIOChannel *channel,
           if (error) {
             g_warning ("Error showing notification: %s", error->message);
             g_error_free (error);
+            error = NULL;
           }
         }
       } else if (!g_ascii_strcasecmp (command, "visible")) {
@@ -233,3 +236,4 @@ zenity_notification (ZenityData *data, ZenityNotificationData *notification_data
 
   gtk_main ();
 }
+#endif
