@@ -76,9 +76,11 @@ static gchar   *zenity_list_print_column;
 static gchar   *zenity_list_hide_column;
 static gboolean zenity_list_hide_header;
 
+#ifdef HAVE_LIBNOTIFY
 /* Notification Dialog Options */
 static gboolean zenity_notification_active;
 static gboolean zenity_notification_listen;
+#endif
 
 /* Progress Dialog Options */
 static gboolean zenity_progress_active;
@@ -95,6 +97,7 @@ static gchar   *zenity_question_cancel_button;
 
 /* Text Dialog Options */
 static gboolean zenity_text_active;
+static gchar   *zenity_text_font;
 
 /* Warning Dialog Options */
 static gboolean zenity_warning_active;
@@ -113,10 +116,24 @@ static gboolean zenity_colorsel_active;
 static gchar   *zenity_colorsel_color;
 static gboolean zenity_colorsel_show_palette;
 
+/* Password Dialog Options */
+static gboolean zenity_password_active;
+static gboolean zenity_password_show_username;
+
+/* Forms Dialog Options */
+static gboolean zenity_forms_active;
+static gchar   *zenity_forms_date_format;
+
 /* Miscelaneus Options */
 static gboolean zenity_misc_about;
 static gboolean zenity_misc_version;
 
+static gboolean
+zenity_forms_callback (const gchar *option_name,
+                       const gchar *value,
+                       gpointer data,
+                       GError **error);
+
 static GOptionEntry general_options[] = {
   {
     "title",
@@ -523,6 +540,7 @@ static GOptionEntry list_options[] = {
   } 
 };
 
+#ifdef HAVE_LIBNOTIFY
 static GOptionEntry notification_options[] = {
   {
     "notification",
@@ -556,6 +574,8 @@ static GOptionEntry notification_options[] = {
   }
 };
 
+#endif
+
 static GOptionEntry progress_options[] = {
   {
     "progress",
@@ -610,7 +630,7 @@ static GOptionEntry progress_options[] = {
     G_OPTION_ARG_NONE,
     &zenity_progress_auto_kill,
     /* xgettext: no-c-format */
-    N_("Kill parent process if cancel button is pressed"),
+    N_("Kill parent process if Cancel button is pressed"),
     NULL
   },
   {
@@ -620,7 +640,7 @@ static GOptionEntry progress_options[] = {
    G_OPTION_ARG_NONE,
    &zenity_progress_no_cancel,
    /* xgettext: no-c-format */
-   N_("Hide cancel button"),
+   N_("Hide Cancel button"),
    NULL
   },
   { 
@@ -707,6 +727,15 @@ static GOptionEntry text_options[] = {
     N_("Allow changes to text"),
     NULL
   },
+  {
+    "font",
+    '\0',
+    0,
+    G_OPTION_ARG_STRING,
+    &zenity_text_font,
+    N_("Set the text font"),
+    N_("TEXT")
+  },
   { 
     NULL 
   }
@@ -823,6 +852,99 @@ static GOptionEntry scale_options[] = {
   }
 };
 
+static GOptionEntry forms_dialog_options[] = {
+  {
+    "forms",
+    '\0',
+    G_OPTION_FLAG_IN_MAIN,
+    G_OPTION_ARG_NONE,
+    &zenity_forms_active,
+    N_("Display forms dialog"),
+    NULL
+  },
+  {
+    "add-entry",
+    '\0',
+    0,
+    G_OPTION_ARG_CALLBACK,
+    zenity_forms_callback,
+    N_("Add a new Entry in forms dialog"),
+    N_("Field name")
+  },
+  {
+    "add-password",
+    '\0',
+    0,
+    G_OPTION_ARG_CALLBACK,
+    zenity_forms_callback,
+    N_("Add a new Password Entry in forms dialog"),
+    N_("Field name")
+  },
+  {
+    "add-calendar",
+    '\0',
+    0,
+    G_OPTION_ARG_CALLBACK,
+    zenity_forms_callback,
+    N_("Add a new Calendar in forms dialog"),
+    N_("Calendar field name")
+  },
+  {
+    "text",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_general_dialog_text,
+    N_("Set the dialog text"),
+    N_("TEXT")
+  },
+  {
+    "separator",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_general_separator,
+    N_("Set output separator character"),
+    N_("SEPARATOR")
+  },
+  {
+    "date-format",
+    '\0',
+    0,
+    G_OPTION_ARG_STRING,
+    &zenity_forms_date_format,
+    N_("Set the format for the returned date"),
+    N_("PATTERN")
+  },
+  {
+    NULL
+  }
+};
+
+static GOptionEntry password_dialog_options[] = {
+  {
+    "password",
+    '\0',
+    G_OPTION_FLAG_IN_MAIN,
+    G_OPTION_ARG_NONE,
+    &zenity_password_active,
+    N_("Display password dialog"),
+    NULL
+  },
+  {
+    "username",
+    '\0',
+    0,
+    G_OPTION_ARG_NONE,
+    &zenity_password_show_username,
+    N_("Display the username option"),
+    NULL
+  },
+  {
+    NULL
+  } 
+};
+
 static GOptionEntry color_selection_options[] = {
   {
     "color-selection",
@@ -899,8 +1021,12 @@ zenity_option_init (void) {
   results->progress_data = g_new0 (ZenityProgressData, 1); 
   results->text_data = g_new0 (ZenityTextData, 1);
   results->tree_data = g_new0 (ZenityTreeData, 1);
+#ifdef HAVE_LIBNOTIFY
   results->notification_data = g_new0 (ZenityNotificationData, 1);
+#endif
   results->color_data = g_new0 (ZenityColorData, 1);
+  results->password_data = g_new0 (ZenityPasswordData, 1);
+  results->forms_data = g_new0 (ZenityFormsData, 1);
 }
 
 void
@@ -918,6 +1044,9 @@ zenity_option_free (void) {
   if (zenity_calendar_date_format)
     g_free (zenity_calendar_date_format);
 
+  if (zenity_forms_date_format)
+    g_free (zenity_forms_date_format);
+
   if (zenity_entry_entry_text)
     g_free (zenity_entry_entry_text);
 
@@ -936,9 +1065,12 @@ zenity_option_free (void) {
   if (zenity_question_cancel_button)
     g_free (zenity_question_cancel_button);
 
+  if (zenity_text_font)
+    g_free (zenity_text_font);
+
   if (zenity_colorsel_color)
     g_free (zenity_colorsel_color);
-
+  
   g_option_context_free (ctx);
 }
 
@@ -965,6 +1097,27 @@ zenity_option_get_name (GOptionEntry *entries, gpointer arg_data)
   return NULL;   
 }
 
+/* Forms callback */
+static gboolean
+zenity_forms_callback (const gchar *option_name,
+                       const gchar *value,
+                       gpointer data,
+                       GError **error)
+{
+  ZenityFormsValue *forms_value = g_new0 (ZenityFormsValue, 1);
+  forms_value->option_value = g_strdup(value);
+  if (g_strcmp0(option_name, "--add-entry") == 0)
+    forms_value->type = ZENITY_FORMS_ENTRY;
+  else if (g_strcmp0(option_name, "--add-calendar") == 0)
+    forms_value->type = ZENITY_FORMS_CALENDAR;
+  else if (g_strcmp0(option_name, "--add-password") == 0)
+    forms_value->type = ZENITY_FORMS_PASSWORD;
+
+  results->forms_data->list = g_slist_append(results->forms_data->list, forms_value);
+
+  return TRUE;
+}
+
 /* Error callback */
 static void 
 zenity_option_error_callback (GOptionContext *context,
@@ -1080,6 +1233,7 @@ zenity_list_pre_callback (GOptionContext *context,
   return TRUE;
 }
 
+#ifdef HAVE_LIBNOTIFY
 static gboolean
 zenity_notification_pre_callback (GOptionContext *context,
 		                  GOptionGroup   *group,
@@ -1091,6 +1245,7 @@ zenity_notification_pre_callback (GOptionContext *context,
 
   return TRUE;
 }
+#endif
 
 static gboolean
 zenity_progress_pre_callback (GOptionContext *context,
@@ -1125,6 +1280,7 @@ zenity_text_pre_callback (GOptionContext *context,
 		          GError        **error)
 {
   zenity_text_active = FALSE;
+  zenity_text_font = NULL;
 
   return TRUE;
 }
@@ -1170,6 +1326,29 @@ zenity_color_pre_callback (GOptionContext *context,
   return TRUE;
 }
 
+static gboolean
+zenity_password_pre_callback (GOptionContext *context,
+                              GOptionGroup   *group,
+                              gpointer        data,
+                              GError        **error)
+{
+  zenity_password_active = FALSE;
+  zenity_password_show_username = FALSE;
+
+  return TRUE; 
+}
+
+static gboolean
+zenity_forms_pre_callback (GOptionContext *context,
+                           GOptionGroup   *group,
+                           gpointer        data,
+                           GError        **error)
+{
+  zenity_forms_active = FALSE;
+  zenity_forms_date_format = NULL;
+  return TRUE;
+}
+
 static gboolean
 zenity_misc_pre_callback (GOptionContext *context,
 		          GOptionGroup   *group,
@@ -1402,6 +1581,7 @@ zenity_list_post_callback (GOptionContext *context,
   return TRUE;
 }
 
+#ifdef HAVE_LIBNOTIFY
 static gboolean
 zenity_notification_post_callback (GOptionContext *context,
 		                   GOptionGroup   *group,
@@ -1421,6 +1601,7 @@ zenity_notification_post_callback (GOptionContext *context,
 
   return TRUE;
 }
+#endif
 
 static gboolean
 zenity_progress_post_callback (GOptionContext *context,
@@ -1492,7 +1673,13 @@ zenity_text_post_callback (GOptionContext *context,
   if (results->mode == MODE_TEXTINFO) {
     results->text_data->uri = zenity_general_uri;
     results->text_data->editable = zenity_general_editable;
-  } 
+    results->text_data->no_wrap = zenity_general_dialog_no_wrap;
+    results->text_data->font = zenity_text_font;
+  } else {
+    if (zenity_text_font)
+      zenity_option_error (zenity_option_get_name (text_options, &zenity_text_font),
+                           ERROR_SUPPORT);
+  }
     
   return TRUE;
 }
@@ -1559,6 +1746,48 @@ zenity_color_post_callback (GOptionContext *context,
   return TRUE;
 }
 
+static gboolean
+zenity_forms_post_callback (GOptionContext *context,
+                            GOptionGroup   *group,
+                            gpointer        data,
+                            GError        **error)
+{
+  zenity_option_set_dialog_mode (zenity_forms_active, MODE_FORMS);
+  if (results->mode == MODE_FORMS) {
+    results->forms_data->dialog_text = zenity_general_dialog_text;
+    results->forms_data->separator = zenity_general_separator;
+    if (zenity_forms_date_format)
+      results->forms_data->date_format = zenity_forms_date_format;
+    else
+      results->forms_data->date_format = g_locale_to_utf8 (nl_langinfo (D_FMT), -1, NULL, NULL, NULL);
+  } else {
+    if (zenity_forms_date_format)
+      zenity_option_error (zenity_option_get_name (forms_dialog_options, &zenity_forms_date_format),
+                           ERROR_SUPPORT);
+  }
+
+  return TRUE;
+}
+
+
+static gboolean
+zenity_password_post_callback (GOptionContext *context,
+                               GOptionGroup   *group,
+                               gpointer        data,
+                               GError        **error)
+{
+  zenity_option_set_dialog_mode (zenity_password_active, MODE_PASSWORD);
+  if (results->mode == MODE_PASSWORD) {
+    results->password_data->username = zenity_password_show_username;
+  } else {
+    if (zenity_password_show_username)
+      zenity_option_error (zenity_option_get_name(password_dialog_options, &zenity_password_show_username),
+                           ERROR_SUPPORT);
+  }
+
+  return TRUE;
+}
+
 static gboolean
 zenity_misc_post_callback (GOptionContext *context,
 		           GOptionGroup   *group,
@@ -1655,7 +1884,8 @@ zenity_create_context (void)
   g_option_group_set_error_hook (a_group, zenity_option_error_callback);
   g_option_group_set_translation_domain (a_group, GETTEXT_PACKAGE);
   g_option_context_add_group(tmp_ctx, a_group);
-  
+ 
+#ifdef HAVE_LIBNOTIFY 
   /* Adds notification option entries */
   a_group = g_option_group_new("notification", 
                                N_("Notification icon options"), 
@@ -1666,6 +1896,7 @@ zenity_create_context (void)
   g_option_group_set_error_hook (a_group, zenity_option_error_callback);
   g_option_group_set_translation_domain (a_group, GETTEXT_PACKAGE);
   g_option_context_add_group(tmp_ctx, a_group);
+#endif
   
   /* Adds progress option entries */
   a_group = g_option_group_new("progress", 
@@ -1733,6 +1964,28 @@ zenity_create_context (void)
   g_option_group_set_translation_domain (a_group, GETTEXT_PACKAGE);
   g_option_context_add_group(tmp_ctx, a_group);
 
+  /* Adds password dialog option entries */
+  a_group = g_option_group_new("password",
+                               N_("Password dialog options"),
+                               N_("Show password dialog options"), NULL, NULL);
+  g_option_group_add_entries (a_group, password_dialog_options);
+  g_option_group_set_parse_hooks (a_group,
+                    zenity_password_pre_callback, zenity_password_post_callback);
+  g_option_group_set_error_hook (a_group, zenity_option_error_callback);
+  g_option_group_set_translation_domain (a_group, GETTEXT_PACKAGE);
+  g_option_context_add_group(tmp_ctx, a_group);
+
+  /* Adds forms dialog option entries */
+  a_group = g_option_group_new("forms",
+                               N_("Forms dialog options"),
+                               N_("Show forms dialog options"), NULL, NULL);
+  g_option_group_add_entries (a_group, forms_dialog_options);
+  g_option_group_set_parse_hooks (a_group,
+                    zenity_forms_pre_callback, zenity_forms_post_callback);
+  g_option_group_set_error_hook (a_group, zenity_option_error_callback);
+  g_option_group_set_translation_domain (a_group, GETTEXT_PACKAGE);
+  g_option_context_add_group(tmp_ctx, a_group);
+
   /* Adds misc option entries */
   a_group = g_option_group_new("misc", 
                                N_("Miscellaneous options"), 
@@ -1796,7 +2049,7 @@ zenity_option_parse (gint argc, gchar **argv)
       zenity_option_error (zenity_option_get_name (calendar_options, &zenity_general_dialog_text), ERROR_SUPPORT);
   
   if (strcmp (zenity_general_separator, "|") != 0)
-    if (results->mode != MODE_LIST && results->mode != MODE_FILE)
+    if (results->mode != MODE_LIST && results->mode != MODE_FILE && results->mode != MODE_FORMS)
       zenity_option_error (zenity_option_get_name (list_options, &zenity_general_separator), ERROR_SUPPORT);
   
   if (zenity_general_multiple)
@@ -1812,7 +2065,7 @@ zenity_option_parse (gint argc, gchar **argv)
       zenity_option_error (zenity_option_get_name (text_options, &zenity_general_uri), ERROR_SUPPORT);
   
   if (zenity_general_dialog_no_wrap)
-    if (results->mode != MODE_INFO && results->mode != MODE_ERROR && results->mode != MODE_QUESTION && results->mode != MODE_WARNING)
+    if (results->mode != MODE_INFO && results->mode != MODE_ERROR && results->mode != MODE_QUESTION && results->mode != MODE_WARNING && results->mode != MODE_TEXTINFO)
       zenity_option_error (zenity_option_get_name (text_options, &zenity_general_dialog_no_wrap), ERROR_SUPPORT);
   
   return results; 
